#include <u.h>
#include <libc.h>
/*
 * "call" is a Plan9 version of "dial" in P9P by Russ Cox.
 * command named "dial" is alreay in 9front.
 * so I avoided the name in porting to 9front.
 *
 * "-t" option is added which was in connect.c
 * "-e" flag is discarded. instead "-a time" is introduced.
 *
 * compile: cc -o call call.c && cp call $home/bin/amd64
 * -Kenar
 */

int atime = 0; /* alarm time */
int tflag = 0;

void
usage(void)
{
	fprint(2, ("usage: call [-t] [-a time] addr\n"
	"-t: test mode\n"
	"-a time: alarm time to finish\n")
	);
	exits("usage");
}

void
killer(void *x, char *msg)
{
	USED(x);
	if(strcmp(msg, "kill") == 0)
		exits(0);
	noted(NDFLT);
}

void
main(int argc, char **argv)
{
	int fd, pid;
	char buf[8192];
	int n, waitforeof;
	char devdir[NETPATHLEN];
	char *lport = nil;
	char *dest;
	double ftime;

	notify(killer);
	waitforeof = 0;
	ARGBEGIN{
	case 'a':
		ftime = atof(EARGF(usage()));
		if(ftime > 0)
			atime = 1000*ftime; /* time in msec */
		break;
	case 't':
		tflag = 1; break;
	default:
		usage();
	}ARGEND

	if(argc != 1)
		usage();

	dest = argv[0];
	if((fd = dial(dest, lport, devdir, nil)) < 0)
		sysfatal("dial: %r");
	if(tflag){
		NetConnInfo *cinfo;
		fprint(2, "connected to %s on %s\n", dest, devdir);
		cinfo = getnetconninfo(nil, fd);
		fprint(2, "remote %s\n", cinfo->raddr);
		fprint(2, "local  %s\n", cinfo->laddr);
		exits(nil);
	}

	switch(pid = fork()){
	case -1:
		sysfatal("fork: %r");
	case 0:
		while((n = read(0, buf, sizeof buf)) > 0)
			if(write(fd, buf, n) < 0)
				break;
		if(1)
			close(fd); /* disconnect. so, make the server read error */
		if(!atime)
			postnote(PNPROC, getppid(), "kill");
		fprint(2,"child: exiting\n");
		exits(nil);
	}

	fprint(2,"parent: waiting\n");
	alarm(atime);
	while((n = read(fd, buf, sizeof buf)) > 0)
		if(write(1, buf, n) < 0)
			break;
	fprint(2,"parent: done\n");
	postnote(PNPROC, pid, "kill");
	waitpid();
	exits(0);
}
